#include <cstring>
#include <iostream>
#include <cstdio>
#include <new>
#include <inttypes.h> 
#include "frame.h"

frame::frame(double time, unsigned char *img, int width, int height, 
	     int frameSize, int format, int a_iFrameNo)
{
  m_dTimestamp = time;

  m_pcImage = new unsigned char[frameSize];

  if (m_pcImage == NULL)
    fprintf(stderr, "*** Allocation failure ***\n");

  memcpy(m_pcImage, img, frameSize);

  m_iWidth = width;
  m_iHeight = height;

  m_iFrameNo = a_iFrameNo;
  m_iFrameSize = frameSize;
  
  isTransEnd = 0;
  transType = NONE;

  isKeyFrame = 0;
}

frame::frame(double time, uint8_t *ybuffer, uint8_t *ubuffer, uint8_t *vbuffer, 
	    int *linesize, int width, int height, int a_iFrameNo)
{
  m_dTimestamp = time;
  int frameSize = width*height + width*height/2;
 
  m_pcImage = new unsigned char[frameSize];
  
  if (m_pcImage == NULL)
    fprintf(stderr, "*** Allocation failure ***\n");  
  
  //ffmpeg AVFrame buffers length are multiple of 32 (= linesize)
  //when copying the buffer contents to m_pcImage, I need to trim
  //the AVFrame buffer to the correct size (width)
  for (int w=0; w != width; w++)
  	for (int h =0; h != height; h++)
  		m_pcImage[w + h*width] = ybuffer[w + h*linesize[0]];
  
  for (int w=0; w != width/2; w++)
  	for (int h =0; h != height/2; h++) {
  		m_pcImage[width*height + w + h*width/2] = ubuffer[w + h*linesize[1]];
  		m_pcImage[width*height + width*height/4 + w + h*width/2] = vbuffer[w + h*linesize[2]];
  	}

  m_iWidth = width;
  m_iHeight = height;

  m_iFrameNo = a_iFrameNo;
  m_iFrameSize = frameSize;
  
  isTransStart = 0;
  isTransEnd = 0;

  isKeyFrame = 0;
  transType = NONE;
}


frame::frame(double time, char *ybuffer, char *ubuffer, char *vbuffer, 
             int width, int height, int frameSize, int format, int a_iFrameNo)
{
  m_dTimestamp = time;
 
  m_pcImage = new unsigned char[frameSize];

  memcpy(m_pcImage, ybuffer, width * height);
  memcpy(m_pcImage+(width*height), ubuffer, (width*height/4));
  memcpy(m_pcImage+(width*height)+(width*height/4), vbuffer, (width*height/4));

  m_iWidth = width;
  m_iHeight = height;

  m_iFrameNo = a_iFrameNo;
  m_iFrameSize = frameSize;
  
  isTransStart = 0;
  isTransEnd = 0;

  isKeyFrame = 0;
  transType = NONE;
}


//Copy constructor
frame::frame(const frame &fr)
{
  m_iFrameNo = fr.m_iFrameNo;
  m_dTimestamp = fr.m_dTimestamp;
  m_iWidth = fr.m_iWidth;
  m_iHeight = fr.m_iHeight;
  m_iFrameSize = fr.m_iFrameSize;
  isTransStart = fr.isTransStart;
  isTransEnd = fr.isTransEnd;
  transStartFr = fr.transStartFr;
  transType = fr.transType;
  isKeyFrame = fr.isKeyFrame;

  m_pcImage = new unsigned char[m_iFrameSize];
  
  memcpy(m_pcImage, fr.m_pcImage, m_iFrameSize);
}

frame::~frame()
{
  delete []m_pcImage;
}

void frame::DeleteImage()
{
  delete []m_pcImage;
}





