#include <new>
#include <signal.h>
#include <string.h> //for memset
#include <sys/time.h>
#include <inttypes.h> 
#include "mpeg_decoder.h"

extern void finishup(int enc_mode);
extern void Fatal(const char *fmt,...);

MPEG_Decoder::MPEG_Decoder(char* filename) : Decoder(filename)
{
  //printf("MPEG_Decoder: Entered constructor\n");
  
//  //Initialize libavcodec and register all the codecs and formats.
//  av_register_all(); //done in main
  
  //Open video file
  int result = av_open_input_file(&pFormatCtx, filename, NULL, 0, NULL);
  //printf("MPEG_Decoder: checked sig\n");
  if (result)
    Fatal("%s not a valid MPEG file", filename);

  // Retrieve stream information
  if(av_find_stream_info(pFormatCtx)<0)
  //printf("MPEG_Decoder: opened file\n");
    Fatal("Failed to open MPEG file: %s", filename);

  // Dump information about file onto standard error
  dump_format(pFormatCtx, 0, filename, false);

  // VIDEO
    
  // Find the first video stream
    videoStream = -1;
    for(i=0; i<pFormatCtx->nb_streams; i++)
        if(pFormatCtx->streams[i]->codec->codec_type==CODEC_TYPE_VIDEO)
        {
            videoStream=i;
            break;
        }
    if(videoStream == -1)
        Fatal("Failed to open MPEG file: %s", filename);// Didn't find a video stream

    // Get a pointer to the codec context for the video stream
    pVCodecCtx=pFormatCtx->streams[videoStream]->codec;

    // Find the decoder for the video stream
    pVCodec=avcodec_find_decoder(pVCodecCtx->codec_id);
    printf("Codec ID: %i\n",pVCodecCtx->codec_id); ////
    if(pVCodec==NULL)
        Fatal("No available codec to process MPEG file: %s", filename);// Codec not found

    // Open codec
    if(avcodec_open(pVCodecCtx, pVCodec)<0)
        Fatal("Could not open codec");// Could not open codec

    // Hack to correct wrong frame rates that seem to be generated by some 
    // codecs
//    if(pVCodecCtx->frame_rate>1000 && pVCodecCtx->frame_rate_base==1)
//       pVCodecCtx->frame_rate_base=1000;
   
   
   // AUDIO
       
   // Find the first audio stream
    audioStream = -1;
    for(i=0; i<pFormatCtx->nb_streams; i++)
        if(pFormatCtx->streams[i]->codec->codec_type==CODEC_TYPE_AUDIO)
        {
            audioStream=i;
            break;
        }
    if(audioStream==-1)
        Fatal("Failed to open MPEG file: %s", filename);// Didn't find an audio stream
    
    //Find the number of audio streams (channels)
    nbAudioStreams=0;
    for(i=0; i<pFormatCtx->nb_streams; i++)
        if(pFormatCtx->streams[i]->codec->codec_type==CODEC_TYPE_AUDIO)
        {
            nbAudioStreams++;
        }
        
    // Get a pointer to the audio codec context for the video stream
    pACodecCtx=pFormatCtx->streams[audioStream]->codec;

        
  int width = pVCodecCtx->width;
  int height = pVCodecCtx->height;
  int length = pFormatCtx->cur_len;
  float framerate = 0;//pVCodecCtx->frame_rate;
  int channels = nbAudioStreams;
  int samplerate = pACodecCtx->sample_rate;
  //printf("MPEG_Decoder: got properties\n");
  vidprops = new VideoProperties(width, height, length, framerate,
                                 channels, samplerate);
  //printf("MPEG_Decoder: all done\n");
  frameno=0;
}

MPEG_Decoder::~MPEG_Decoder()
{
  delete vidprops;
  if (currframe)
    delete currframe;
//  av_free(pFrame);
  avcodec_close(pVCodecCtx);
  //avcodec_close(pACodecCtx);
  av_close_input_file(pFormatCtx);
}

VideoProperties* MPEG_Decoder::GetVideoProperties()
{
  return vidprops;
}

void timer_handler (int signum) 
{ 
  finishup(1);
}

//Decodes one YUV frame; returns 0 if last frame, 1 otherwise
int MPEG_Decoder::GetFrame(frame* &vidframe)
{
  //set up a timeout alarm
  struct sigaction sa; 
  struct itimerval timer; 
  memset (&sa, 0, sizeof (sa)); 
  sa.sa_handler = &timer_handler; 
  sigaction (SIGALRM, &sa, NULL); 
  /* Configure the timer to expire after 10sec... */ 
  timer.it_value.tv_sec = 10; 
  timer.it_value.tv_usec = 0; 
  timer.it_interval.tv_sec = 0; 
  timer.it_interval.tv_usec = 0; 

  setitimer (ITIMER_REAL, &timer, NULL);

  int len=0, got_picture;

  //Allocate a YUV frame
  AVFrame *pFrame = avcodec_alloc_frame();
  AVPacket pkt;
  uint8_t *ybuffer, *ubuffer, *vbuffer;
  
  if (currframe)
  {
  delete currframe;
  currframe = 0;
  }

  int width = vidprops->GetWidth();
  int height = vidprops->GetHeight();

  // Decode proper
  while(1){	
    //printf("read %i\n",frameno);
	
	// Read a frame/packet.
	if(av_read_frame(pFormatCtx, &pkt) < 0 ) {
		//printf("end picture %i\n",frameno);
    	av_free(pFrame);
    	//av_free_packet(&pkt);
    	return 0;
	}

	// If it's a video packet from our video stream... 
	if(pkt.stream_index==videoStream){

	    // Decode the video
	    len = avcodec_decode_video(pVCodecCtx, pFrame, &got_picture, 
					pkt.data, pkt.size);

	    if (got_picture) {
	    	//printf("got picture %i\n",frameno);
	    	//printf("got picture %i %i\n",frameno,pFrame->key_frame);////
	    	ybuffer = pFrame->data[0];
  			ubuffer = pFrame->data[1];
  			vbuffer = pFrame->data[2];
  			double timestamp = 0;//(double)frameno*1000/(pVCodecCtx->frame_rate);
  			currframe = new frame(timestamp, ybuffer, ubuffer, vbuffer,
                        pFrame->linesize, width, height, frameno);
            vidframe = currframe;

    		av_free(pFrame);
    		av_free_packet(&pkt);
            frameno++;
	    	return 1;
	    }
	 }
	    //av_free(pFrame);
    	av_free_packet(&pkt);
   }
}

//int MPEG_Decoder::GetAudio(short* buffer, int samples, int channel)
//{
  //Audio
//  uint8_t *audio_buf;
//  int16_t *outbuf;
//  int out_size, size;
//  int result;
//  if (!channel)
//		//Decode the audio			
//		lena = avcodec_decode_audio(pACodecCtx, outbuf, &out_size, audio_buf, size);
//		printf("%i\n", lena);
//    result = mpeg3_read_audio(mpegfile, NULL, buffer, channel, samples, 0);
//  else
//    result = mpeg3_reread_audio(mpegfile, NULL, buffer, channel, samples, 0);
//
//  return result;
//}


double MPEG_Decoder::GetPercentageDone()
{
    return 0; //  return mpeg3_tell_percentage(mpegfile);
}
